"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var either_1 = require("./either");
var chai = require("chai");
var expect = chai.expect;
var createError = function (value) { return "Value \"" + value + "\" is not an Either type"; };
var add1 = function (n) { return n + 1; };
var removeFirstElement = function (arr) {
    if (arr.length === 0) {
        throw new Error('Array is empty');
    }
    return arr.slice(1);
};
describe('Either', function () {
    describe('isRight', function () {
        it('should return false when Left is provided', function () {
            expect(either_1.isRight(either_1.left(''))).to.be.false;
        });
        it('should return true when Right is provided', function () {
            expect(either_1.isRight(either_1.right('hola'))).to.be.true;
        });
        it('should throw when a non Either type is provided', function () {
            expect(function () { return either_1.isRight([1, 2, 3]); }).to.throw(createError([1, 2, 3]));
        });
        it('should throw when null or undefined is provided', function () {
            expect(function () { return either_1.isRight(null); }).to.throw(createError(null));
            expect(function () { return either_1.isRight(undefined); }).to.throw(createError(undefined));
        });
    });
    describe('isLeft', function () {
        it('should return false when Right is provided', function () {
            expect(either_1.isLeft(either_1.right('hola'))).to.be.false;
        });
        it('should return true when Left is provided', function () {
            expect(either_1.isLeft(either_1.left('hola'))).to.be.true;
        });
        it('should throw when a non Either type is provided', function () {
            expect(function () { return either_1.isLeft('lala'); }).to.throw(createError('lala'));
        });
        it('should throw when null or undefined is provided', function () {
            expect(function () { return either_1.isLeft(null); }).to.throw(createError(null));
            expect(function () { return either_1.isLeft(undefined); }).to.throw(createError(undefined));
        });
    });
    describe('withDefault', function () {
        it('should return the Right value when Right is provided', function () {
            expect(either_1.withDefault(either_1.right(6), 0)).to.equal(6);
        });
        it('should return the default value when Left is provided', function () {
            expect(either_1.withDefault(either_1.left('err'), 0)).to.equal(0);
        });
        it('should throw when a non Either type is provided', function () {
            expect(function () { return either_1.withDefault('hola', 2); }).to.throw(createError('hola'));
        });
        it('should throw when null or undefined is provided', function () {
            expect(function () { return either_1.withDefault(null, 1); }).to.throw(createError(null));
            expect(function () { return either_1.withDefault(undefined, 1); }).to.throw(createError(undefined));
        });
    });
    describe('map', function () {
        it('should return Right when mapping over Right', function () {
            var four = either_1.right(4);
            var five = either_1.map(add1, four);
            expect(either_1.isRight(five)).to.be.true;
        });
        it('should return Left when mapping over Left', function () {
            var result = either_1.map(add1, either_1.left('err'));
            expect(either_1.isLeft(result)).to.be.true;
        });
        it('should return 5 when adding 1 to Right(4)', function () {
            var four = either_1.right(4);
            var five = either_1.map(add1, four);
            expect(either_1.isRight(five)).to.be.true;
            expect(either_1.withDefault(five, 0)).to.equal(5);
        });
        it('should remove the first element of [1,2,3] succesfully', function () {
            var list = either_1.map(removeFirstElement, either_1.right([1, 2, 3]));
            expect(either_1.isRight(list)).to.be.true;
            expect(either_1.withDefault(list, [])).to.eql([2, 3]);
        });
        it('should return a Left when mapping produces an exception', function () {
            var list = either_1.map(removeFirstElement, either_1.right([]));
            expect(either_1.isLeft(list)).to.be.true;
            expect(either_1.withDefault(list, ['default', 'val'])).to.eql(['default', 'val']);
        });
        it('should throw when mapping over null, undefined or any non Either type', function () {
            expect(function () { return either_1.map(add1, null); }).to.throw(createError(null));
            expect(function () { return either_1.map(add1, undefined); }).to.throw(createError(undefined));
            expect(function () { return either_1.map(add1, {}); }).to.throw(createError('[object Object]'));
        });
    });
    describe('andThen', function () {
        var removeFirstLifted = function (arr) {
            try {
                return either_1.right(removeFirstElement(arr));
            }
            catch (error) {
                return either_1.left(error);
            }
        };
        it('should perform chained Right transformations', function () {
            var result = either_1.andThen(function (arr) { return removeFirstLifted(arr); }, removeFirstLifted(['a', 'b', 'c']));
            expect(either_1.withDefault(result, ['default val'])).to.eql(['c']);
        });
        it('should perform chained Right transformations and fail with Left', function () {
            var result = either_1.andThen(function (arr) { return either_1.andThen(function (arr2) { return removeFirstLifted(arr2); }, removeFirstLifted(arr)); }, removeFirstLifted(['a', 'b']));
            expect(either_1.withDefault(result, ['default val'])).to.eql(['default val']);
        });
        it('should throw when chaining over null, undefined or any non Either type', function () {
            expect(function () { return either_1.andThen(function (a) { return either_1.right(a); }, null); }).to.throw(createError(null));
            expect(function () { return either_1.andThen(function (a) { return either_1.right(a); }, undefined); }).to.throw(createError(undefined));
            expect(function () { return either_1.andThen(function (a) { return either_1.right(a); }, {}); }).to.throw(createError('[object Object]'));
        });
    });
    describe('caseOf', function () {
        it('should Launch 5 missiles', function () {
            var result = either_1.caseOf({
                Left: function (err) { return "Error: " + err; },
                Right: function (n) { return "Launch " + n + " missiles"; }
            }, either_1.right('5'));
            expect(result).to.equal('Launch 5 missiles');
        });
        it('should zzz', function () {
            var result = either_1.caseOf({
                Left: function () { return 'zzz'; },
                Right: function (n) { return "Launch " + n + " missiles"; }
            }, either_1.left('zzz'));
            expect(result).to.equal('zzz');
        });
    });
    describe('examples', function () {
        it('should work ', function () {
            var head = function (arr) {
                if (arr.length === 0) {
                    throw new Error('Array is empty');
                }
                return arr.slice(0, 1)[0];
            };
            var num = either_1.map(head, either_1.right([99, 109, 22, 65]));
            expect(either_1.withDefault(either_1.map(add1, num), 0)).to.equal(100);
            expect(either_1.withDefault(either_1.andThen(function (n) { return either_1.right(add1(n)); }, num), 0)).to.equal(100);
        });
    });
});
